/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2002 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/BlueDiamond/studio/RCS/ProPathCmp.html,v 1.1 2002/04/20 14:17:46 sauge Exp sauge $" NO-UNDO.

{PostFix.i}
{BaseName.i}

DEF INPUT PARAMETER cApplication  AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER cApplPath     AS CHARACTER NO-UNDO.

RUN Logger.p (5, "Hello!", "scott.conf").
RUN Logger.p (3, "Application=" + cApplication, "scott.conf").
RUN Logger.p (3, "PROPATH=" + cApplPath, "scott.conf").

DEF TEMP-TABLE SubTree
  FIELD DirName AS CHARACTER.

DEF VAR iNumApplDir AS INTEGER NO-UNDO.
DEF VAR iCurApplDir AS INTEGER NO-UNDO.

/***************************************************************************/
/* First we break apart the APPLPATH into individual directory entries     */
/***************************************************************************/

ASSIGN iNumApplDir = NUM-ENTRIES(cApplPath).
DO iCurApplDir = 1 TO iNumApplDir:

  CREATE SubTree.
  ASSIGN SubTree.DirName = ENTRY(iCurApplDir, cApplPath).

  /* Find the subdirectories */

  RUN Logger.p (5, "Diving into directory" + SubTree.DirName, "scott.conf").
  RUN ipFindDir (INPUT SubTree.DirName).

END.  /* DO iCurApplDir = 1 TO iNumApplDir */

/* For some reason we end up with an empty record, blow that away */

FOR EACH SubTree EXCLUSIVE-LOCK
WHERE SubTree.DirName = "":
  DELETE SubTree.
END.

/***************************************************************************/
/* In each directory, compile the .p files                                 */
/***************************************************************************/

FOR EACH SubTree NO-LOCK:

  
  RUN Logger.p (5, "Calling ipGenXRef with " + SubTree.DirName, "scott.conf").
  RUN ipGenXRef (INPUT SubTree.DirName, INPUT cApplication).

END. /* FOR EACH SubTree */

/* ------------------------- Internal Procedure --------------------------- */

/* Find all the directories of the APPLPATH as well the subtrees in it */
/* We dont do the PROPATH cuz it might compile some Progress stuff ac- */
/* cidently.                                                           */

PROCEDURE ipFindDir:

  DEF INPUT PARAMETER cApplPath AS CHARACTER NO-UNDO.

  DEF VAR cDirList AS CHARACTER NO-UNDO.
  DEF VAR cDummy   AS CHARACTER NO-UNDO.
  DEF VAR iNumApplDir AS INTEGER NO-UNDO.
  DEF VAR iCurApplDir AS INTEGER NO-UNDO.

  DEF VAR cDelimiter AS CHARACTER NO-UNDO.

  ASSIGN cDelimiter = CHR(250).

  /* Get a listing of entries, we keep the type directory */

  RUN ReadDirectory.p (INPUT cApplPath, OUTPUT cDirList, OUTPUT cDummy).

  /* For each directory entry, we make a SubTree record, then call into */
  /* that directory for more sub-directories.                           */

  ASSIGN iNumApplDir = NUM-ENTRIES(cDirList, cDelimiter).
  DO iCurApplDir = 1 TO iNumApplDir:

    CREATE SubTree.
    ASSIGN SubTree.DirName = cApplPath + 
                             "/" + 
                             ENTRY(iCurApplDir, cDirList, cDelimiter).
    RUN ipFindDir (INPUT SubTree.DirName).

  END.  /* DO iCurApplDir = 1 TO iNumApplDir */

END. /* PROCEDURE ipFindDir */

PROCEDURE ipGenXRef:

  DEF INPUT PARAMETER cDirName AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER cApplication AS CHARACTER NO-UNDO.

  DEF VAR cFileList AS CHARACTER NO-UNDO.
  DEF VAR iCurFileEntry AS INTEGER NO-UNDO.
  DEF VAR iNumFileEntry AS INTEGER NO-UNDO.
  DEF VAR cDummy AS CHARACTER NO-UNDO.
  DEF VAR cInputFile AS CHARACTER NO-UNDO.
  DEF VAR cDelimiter AS CHARACTER NO-UNDO.
  DEF VAR cXRefFileName AS CHARACTER NO-UNDO.

  ASSIGN cDelimiter = CHR(250).

  RUN ReadDirectory.p (INPUT cDirName, OUTPUT cDummy, OUTPUT cFileList).

  ASSIGN iNumFileEntry = NUM-ENTRIES(cFileList, cDelimiter).
  DO iCurFileEntry = 1 TO iNumFileEntry:

     ASSIGN cInputFile = ENTRY(iCurFileEntry, cFileList, cDelimiter).
     IF PostFix(cInputFile) = ".p" OR PostFix(cInputFile) = ".w" THEN DO: 

       ASSIGN cInputFile = cDirName + "/" + cInputFile.

       RUN Logger.p (5, "XRef'ing " + cInputFile, "scott.conf").
       RUN MkXRef.p (INPUT cInputFile, OUTPUT cXRefFileName). 
       RUN Logger.p (5, "Loading " + cXrefFileName, "scott.conf").
       RUN Logger.p (5, "Application=" + cApplication, "scott.conf").
       RUN LoadXRef.p (INPUT cApplication, INPUT cXRefFileName). 

     END.

  END.

END.

